#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/29

"""
__all__ = [
    'file_write_csv_list_list',
    'file_write_csv_dict_list',
    'file_write_csv_list_dict',
    'file_write_csv_dict_dict',
    'file_read_csv_list_list',
    'file_read_csv_dict_list',
    'file_read_csv_list_dict',
    'file_read_csv_dict_dict',
]


def file_write_csv_list_list(path: str, input_content: list, split=',', split_replace='_', encoding='utf8'):
    split, split_replace, encoding = map(str, [split, split_replace, encoding])
    with open(path, mode='w', encoding=encoding) as file:
        for i, v in enumerate(input_content):
            if i > 0:
                file.write('\n')
            for j, w in enumerate(v):
                if j > 0:
                    file.write(split)
                file.write(str(w).replace(split, split_replace))


def file_write_csv_dict_list(path: str, input_content: dict, split=',', split_replace='_', encoding='utf8'):
    split, split_replace, encoding = map(str, [split, split_replace, encoding])
    with open(path, mode='w', encoding=encoding) as file:
        for i, v in input_content.items():
            if i > 0:
                file.write('\n')
            file.write(str(i))
            for j, w in enumerate(v):
                file.write(split)
                file.write(str(w).replace(split, split_replace))


def file_write_csv_list_dict(path: str, input_content: list, default='', split=',', split_replace='_', encoding='utf8'):
    default, split, split_replace, encoding = map(str, [default, split, split_replace, encoding])
    with open(path, mode='w', encoding=encoding) as file:
        j_list = []
        for i, v in enumerate(input_content):
            for j, w in v.items():
                if j not in j_list:
                    j_list.append(j)
        for i, v in enumerate(input_content):
            if i > 0:
                file.write('\n')
            for j_index, j in j_list:
                if j_index > 0:
                    file.write(split)
                w = v.get(j, default)
                file.write(str(w).replace(split, split_replace))


def file_write_csv_dict_dict(path: str, input_content: dict, default='', title='key', split=',', split_replace='_', encoding='utf8'):
    default, title, split, split_replace, encoding = map(str, [default, title, split, split_replace, encoding])
    with open(path, mode='w', encoding=encoding) as file:
        j_list = []
        for i, v in input_content.items():
            for j, w in v.items():
                if j not in j_list:
                    j_list.append(j)
        file.write(default.join([title] + j_list))
        for i, v in input_content.items():
            file.write('\n' + str(i))
            for j_index, j in j_list:
                file.write(split)
                w = v.get(j, default)
                file.write(str(w).replace(split, split_replace))


def file_read_csv_list_list(path: str, split=',', ignore_blank_row=True, encoding='utf8'):
    re_list = []
    split, encoding = map(str, [split, encoding])
    with open(path, mode='r', encoding=encoding) as file:
        file_lines = file.readlines()
    for line_one in file_lines:
        temp_line = line_one.strip('\n')
        temp_list = temp_line.split(split)
        if len(temp_list) == 1 and temp_list[0] == '':
            if ignore_blank_row:
                continue
            else:
                temp_list = []
        re_list.append(temp_list)
    return re_list


def file_read_csv_dict_list(path: str, split=',', ignore_blank_row=True, encoding='utf8'):
    re_dict = {}
    split, encoding = map(str, [split, encoding])
    with open(path, mode='r', encoding=encoding) as file:
        file_lines = file.readlines()
    for line_one in file_lines:
        temp_line = line_one.strip('\n')
        temp_list = temp_line.split(split)
        if len(temp_list) == 1:
            if ignore_blank_row or temp_list[0] == '':
                continue
        key = str(temp_list.pop(0))
        re_dict[key] = temp_list
    return re_dict


def file_read_csv_list_dict(path: str, split=',', ignore_blank_row=True, ignore_key_value='', encoding='utf8'):
    re_list = []
    split, encoding = map(str, [split, encoding])
    with open(path, mode='r', encoding=encoding) as file:
        file_lines = file.readlines()
    keys = file_lines[0].strip('\n').split(split)
    for line_one in file_lines[1:]:
        temp_list = line_one.strip('\n').split(split)
        if len(temp_list) == 1 and temp_list[0] == '':
            if ignore_blank_row:
                continue
            else:
                temp_list = []
        temp_dict = {}
        for i_key, v_key in enumerate(keys):
            if i_key < len(temp_list):
                v_value = temp_list[i_key]
                if v_value != ignore_key_value:
                    temp_dict[v_key] = v_value
            else:
                break
        re_list.append(temp_dict)
    return re_list


def file_read_csv_dict_dict(path: str, split=',', ignore_blank_row=True, ignore_key_value='', encoding='utf8'):
    re_dict = {}
    split, encoding = map(str, [split, encoding])
    with open(path, mode='r', encoding=encoding) as file:
        file_lines = file.readlines()
    keys = file_lines[0].strip('\n').split(split)[1:]
    for line_one in file_lines[1:]:
        temp_list = line_one.strip('\n').split(split)
        if len(temp_list) == 1 and temp_list[0] == '':
            if ignore_blank_row or temp_list[0] == '':
                continue
        key = str(temp_list.pop(0))
        temp_dict = {}
        for i_key, v_key in enumerate(keys):
            if i_key < len(temp_list):
                v_value = temp_list[i_key]
                if v_value != ignore_key_value:
                    temp_dict[v_key] = v_value
            else:
                break
        re_dict[key] = temp_dict
    return re_dict


if __name__ == '__main__':
    pass



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/24

"""
from .mv_csv import *
from .mv_excel import *

if __name__ == '__main__':
    pass



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/23

"""
import os
import traceback
from tkinter import filedialog

from ..mv_import import MoVoidImport

try:
    import openpyxl
except ModuleNotFoundError as err:
    MoVoidImport.error(err)
else:
    MoVoidImport.version(openpyxl)


def copy_files():
    names = filedialog.askopenfilenames()
    all_len = len(names)
    for i, v in enumerate(names):
        try:
            path, postfix = os.path.splitext(v)
            new_path = path + '_copy' + postfix
            if postfix == '.xlsx':
                wb = openpyxl.load_workbook(v)
                wb.save(new_path)
                print_text = 'excel file success'
            elif postfix == '.csv':
                with open(v, mode='r', encoding='utf8') as file:
                    temp = file.read()
                with open(new_path, mode='w', encoding='utf8') as file:
                    file.write(temp)
                print_text = 'csv file success'
            else:
                print_text = 'unknown file'
        except:
            print_text = 'error!'
            traceback.print_exc()
        finally:
            print('[{}/{} {:.2%}]{}:{}'.format(i, all_len, i / all_len, print_text, v))


if __name__ == '__main__':
    copy_files()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/18

"""
__all__ = [
    'config_update',
    'config_update_regex_rule'
]

import json
import os
import re
from configparser import ConfigParser


def config_update(path: str, ori_config: dict, encoding='utf8'):
    cf1 = ConfigParser()
    if os.path.isfile(path):
        cf1.read(path, encoding)
    cf2 = ConfigParser()
    for i, v in ori_config.items():
        if not cf1.has_section(i):
            cf1.add_section(i)
        cf2.add_section(i)
        for j, w in v.items():
            if cf1.has_option(i, j):
                cf_value = cf1.get(i, j)
                if isinstance(w, int):
                    try:
                        final_str = cf_value
                        final_value = int(cf_value)
                    except:
                        final_str = str(w)
                        final_value = w
                elif isinstance(w, float):
                    try:
                        final_str = cf_value
                        final_value = float(cf_value)
                    except:
                        final_str = str(w)
                        final_value = w
                elif isinstance(w, list):
                    final_str = cf_value
                    final_value = cf_value.split(',')
                elif isinstance(w, dict):
                    final_str = cf_value
                    temp_value = cf_value.split(',')
                    final_value = {}
                    for k in temp_value:
                        k1, k2 = k.split(':')
                        final_value[k1] = k2
                else:
                    final_str = cf_value
                    final_value = cf_value
            else:
                final_str = w
                final_value = w
            cf2.set(i, j, final_str)
            ori_config[i][j] = final_value
    with open(path, 'w', encoding=encoding) as file:
        cf2.write(file)


def config_update_regex_rule(path: str, rule_dict: dict, ori_config: dict, encoding='utf8'):
    cf1 = ConfigParser()
    if os.path.isfile(path):
        cf1.read(path, encoding)
    re_dict = {}
    cf2 = ConfigParser()
    rule_has = {}
    for i_section in cf1.sections():
        v_section = cf1[i_section]
        for i_rule, v_rule in rule_dict.items():
            if re.match(i_rule + '$', i_section):
                re_dict[i_section] = {}
                cf2.add_section(i_section)
                rule_has[i_rule] = True
                for j_rule, w_rule in v_rule.items():
                    if not (j_rule.startswith('__') and j_rule.endswith('__')):
                        temp_str, temp_value = get_value_from_file_rule_ori(v_section.get(j_rule), w_rule, ori_config.get(i_section, {}).get(j_rule))
                        if temp_value is not None:
                            cf2.set(i_section, j_rule, temp_str)
                            re_dict[i_section][j_rule] = temp_value
    for i_rule, v_rule in rule_dict.items():
        if v_rule.get('__must__') and not (i_rule in rule_has and rule_has[i_rule]) and i_rule not in re_dict:
            re_dict[i_rule] = {}
            cf2.add_section(i_rule)
            for j_rule, w_rule in v_rule.items():
                if not (j_rule.startswith('__') and j_rule.endswith('__')):
                    temp_str, temp_value = get_value_from_file_rule_ori(None, w_rule, ori_config.get(i_rule, {}).get(j_rule))
                    if temp_value is not None:
                        cf2.set(i_rule, j_rule, temp_str)
                        re_dict[i_rule][j_rule] = temp_value
    ori_config.clear()
    ori_config.update(re_dict)
    with open(path, mode='w', encoding=encoding) as file:
        cf2.write(file)


def get_value_from_file_rule_ori(file_value, rule, ori_value):
    re_str = None
    re_value = None
    if file_value is None:
        if ori_value is None:
            if rule.get('default') is not None:
                re_value = rule.get('default')
        else:
            re_value = ori_value
    else:
        re_str = file_value
    if re_str is None and re_value is not None:
        re_str = str(re_value)
    if re_str is not None and re_value is None:
        rule_type = rule['type']
        if rule_type == 'int':
            try:
                re_value = int(re_str)
            except:
                re_str, re_value = get_value_from_file_rule_ori(None, rule, ori_value)
        elif rule_type == 'float':
            try:
                re_value = float(re_str)
            except:
                re_str, re_value = get_value_from_file_rule_ori(None, rule, ori_value)
        elif rule_type == 'json':
            try:
                re_value = json.loads(re_str)
            except:
                re_str, re_value = get_value_from_file_rule_ori(None, rule, ori_value)
        elif rule_type == 'bool':
            try:
                re_value = False if re_str.lower() in ('false', 'f', '') else True
            except:
                re_str, re_value = get_value_from_file_rule_ori(None, rule, ori_value)
        elif rule_type == 'list':
            try:
                re_value = re_str.split(',')
            except:
                re_str, re_value = get_value_from_file_rule_ori(None, rule, ori_value)
        elif rule_type == 'list-z':
            try:
                re_value = [_ for _ in re_str.split(',') if _]
            except:
                re_str, re_value = get_value_from_file_rule_ori(None, rule, ori_value)
        elif rule_type == 'dict':
            try:
                temp_list = re_str.split(',')
                re_value = {}
                for i in temp_list:
                    temp_key, temp_value = i.split(':', 1)
                    re_value[temp_key] = temp_value
            except:
                re_str, re_value = get_value_from_file_rule_ori(None, rule, ori_value)
        else:
            re_value = re_str
    return re_str, re_value


if __name__ == '__main__':
    a = {
        'all': {
            '__must__': True,
            'this1': {
                'type': 'str',
                'default': 'doit'
            }
        }
    }
    b = {}
    config_update_regex_rule('config.ini', a, b)
    print('a', a)
    print('b', b)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/18

"""
import datetime
import os
import sys
import time
import traceback
from functools import wraps

from ..movoid_config import *

__all__ = [
    'log_init',
    'log_class_label',
    'log_function_label',
    'log_print',
    'log_debug',
    'log_info',
    'log_warning',
    'log_error',
    'log_critical',
]


def log_init(config_path='config.ini', encoding='utf-8'):
    now_config = {}
    config_update_regex_rule(path=config_path, rule_dict=MoVoidLog.config_rule, ori_config=now_config, encoding=encoding)
    for path, kwargs in now_config.items():
        MoVoidLog.new_log_file(**kwargs)


def log_class_label(label: str = '__default__', level=0, error_level=3):
    if label not in MoVoidLog.log_file:
        if '__default__' in MoVoidLog.log_file:
            label = '__default__'
        else:
            if MoVoidLog.log_file:
                key = list(MoVoidLog.log_file.keys())[0]
                MoVoidLog.reset_default(key)
                label = key
            else:
                raise Exception('there is no log file set,please set one log_file to initialize!')

    def decoration(cls):
        for func_name in dir(cls):
            if not (func_name.startswith('__') and func_name.endswith('__')):
                func = getattr(cls, func_name)
                if callable(func):
                    log_level = getattr(func, 'log_level', level)
                    log_error_level = getattr(func, 'log_error_level', error_level)

                    @wraps(func)
                    def wrapping(*args, **kwargs):
                        sign = time.time()
                        log_text = '[{}]<{}><{}>{},{}'.format(sign, cls.__name__, func_name, args, kwargs)
                        MoVoidLog.write(key=label, text=log_text, level=log_level)
                        try:
                            re_func = func(*args, **kwargs)
                        except Exception as err:
                            log_text = '[{}]<{}><{}>{}'.format(sign, cls.__name__, func_name, traceback.print_exc())
                            MoVoidLog.write(key=label, text=log_text, level=log_error_level)
                        else:
                            log_text = '[{}]<{}><{}>{}'.format(sign, cls.__name__, func_name, re_func)
                            MoVoidLog.write(key=label, text=log_text, level=log_level)

                    setattr(cls, func_name, wrapping)

    return decoration


def log_class_function_level(level=1, error_level=3):
    def decoration(func):
        setattr(func, 'log_level', level)
        setattr(func, 'log_error_level', error_level)

    return decoration


def log_function_label(label: str = '__default__', level=0, error_level=3):
    if label not in MoVoidLog.log_file:
        if '__default__' in MoVoidLog.log_file:
            label = '__default__'
        else:
            if MoVoidLog.log_file:
                key = list(MoVoidLog.log_file.keys())[0]
                MoVoidLog.reset_default(key)
                label = key
            else:
                raise Exception('there is no log file set,please set one log_file to initialize!')

    def decoration(func):
        @wraps(func)
        def wrapping(*args, **kwargs):
            sign = time.time()
            log_text = '[{}]<{}>{},{}'.format(sign, func.__name__, args, kwargs)
            MoVoidLog.write(key=label, text=log_text, level=level)
            try:
                re_func = func(*args, **kwargs)
            except Exception as err:
                log_text = '[{}]<{}>{}'.format(sign, func.__name__, traceback.print_exc())
                MoVoidLog.write(key=label, text=log_text, level=error_level)
            else:
                log_text = '[{}]<{}>{}'.format(sign, func.__name__, re_func)
                MoVoidLog.write(key=label, text=log_text, level=level)

        return wrapping

    return decoration


def log_print(text, level=1, label: str = '__default__', print_out=False):
    if label not in MoVoidLog.log_file:
        if '__default__' in MoVoidLog.log_file:
            label = '__default__'
        else:
            if MoVoidLog.log_file:
                key = list(MoVoidLog.log_file.keys())[0]
                MoVoidLog.reset_default(key)
                label = key
            else:
                raise Exception('there is no log file set,please set one log_file to initialize!')
    MoVoidLog.write(label, text, level, print_out)


def log_debug(text, label: str = '__default__', print_out=False):
    log_print(text, 0, label, print_out)


def log_info(text, label: str = '__default__', print_out=False):
    log_print(text, 1, label, print_out)


def log_warning(text, label: str = '__default__', print_out=False):
    log_print(text, 2, label, print_out)


def log_error(text, label: str = '__default__', print_out=False):
    log_print(text, 3, label, print_out)


def log_critical(text, label: str = '__default__', print_out=False):
    log_print(text, 4, label, print_out)


class MoVoidLog:
    log_file = {}
    log_level = {
        0: 'DEBUG',
        1: 'INFO',
        2: 'WARNING',
        3: 'ERROR',
        4: 'CRITICAL',
    }
    config_rule = {
        'log-.*': {
            '__must__': True,
            'path': {
                'type': 'str',
                'default': 'log'
            },
            'alias': {
                'type': 'list-z',
                'default': []
            },
            'default': {
                'type': 'bool',
                'default': True
            },
            'number': {
                'type': 'int',
                'default': 10
            },
            'size': {
                'type': 'int',
                'default': 20
            },
            'day': {
                'type': 'int',
                'default': 7
            },
            'encoding': {
                'type': 'str',
                'default': 'utf-8'
            },
        }
    }

    @classmethod
    def new_log_file(cls, path: str, alias=None, default=True, number=10, size=20, day=7, encoding: str = 'utf-8'):
        if path not in cls.log_file:
            cls.log_file[path] = MoVoidLogOne(path, number=number, size=size, day=day, encoding=encoding)
            try:
                if alias:
                    for i in alias:
                        try:
                            if i and i in cls.log_file:
                                continue
                            else:
                                cls.log_file[i] = cls.log_file[path]
                        except:
                            continue
                if default:
                    cls.log_file['__default__'] = cls.log_file[path]
            except:
                pass

    @classmethod
    def reset_default(cls, key):
        if key in cls.log_file:
            cls.log_file['__default__'] = cls.log_file[key]

    @classmethod
    def write(cls, key, text, level=0, print_out=False):
        if key in cls.log_file:
            cls.log_file[key].write(text, level, print_out)
        else:
            raise Exception('wrong key:<{}>'.format(key))


class MoVoidLogOne:

    def __init__(self, path, number=10, size=20, day=7, encoding='utf-8'):
        self.dir, self.name = os.path.split(os.path.abspath(path))
        self.number = number
        self.size = size
        self.day = day
        self.encoding = encoding
        self.file_path = None
        self.file = None
        self.date = None
        self.index = -1
        self.find_file()

    def check_file_delete(self):
        all_list = os.listdir(self.dir)
        file_list = []
        for i in all_list:
            this_path = os.path.join(self.dir, i)
            if os.path.isfile(this_path) and i.startswith(self.name + '-') and len(i) >= len(self.name) + 17:
                try:
                    this_date = time.mktime(time.strptime(i[len(self.name) + 1:len(self.name) + 11], "%Y-%m-%d"))
                    this_index = int(i[len(self.name) + 12:-4])
                    if this_date > self.day * 86400:
                        os.remove(this_path)
                    else:
                        file_list.append([this_path, this_date, this_index])
                except:
                    continue
        if len(file_list) > self.number:
            file_list.sort(key=lambda x: x[2])
            file_list.sort(key=lambda x: x[1])
            for i in range(len(file_list) - self.number):
                try:
                    os.remove(file_list[i][0])
                except:
                    continue

    def find_file(self):
        all_list = os.listdir(self.dir)
        prefix = '{}-{}'.format(self.name, time.strftime("%Y-%m-%d", time.localtime(time.time())))
        max_index = -1
        last_path = None
        for i in all_list:
            this_path = os.path.join(self.dir, i)
            if os.path.isfile(this_path) and i.startswith(prefix) and i.endswith('.log'):
                if len(prefix) + 5 >= len(i) >= len(prefix) + 4:
                    this_index = 0
                elif len(i) > len(prefix) + 5:
                    try:
                        this_index = int(i[len(prefix) + 1:-4])
                    except:
                        continue
                else:
                    continue
                if this_index > max_index:
                    max_index = this_index
                    last_path = this_path
        if last_path:
            last_size = os.path.getsize(last_path)
            if last_size >= self.size * 1048576:
                self.file_path = '{}.{}.log'.format(prefix, max_index + 1)
                self.file = open(self.file_path, mode='w', encoding=self.encoding)
            else:
                self.file_path = last_path
                self.file = open(self.file_path, mode='a', encoding=self.encoding)
        else:
            self.file_path = '{}.{}.log'.format(prefix, 0)
            self.file = open(self.file_path, mode='w', encoding=self.encoding)
        self.check_file_delete()

    def write(self, text: str = '', level=0, print_out=False):
        now_time = datetime.datetime.now()
        now_date = now_time.strftime("%Y-%m-%d")
        if now_date == self.date:
            this_size = os.path.getsize(self.file_path)
            if this_size >= self.size * 1048576:
                self.index += 1
                self.create_new_file()
        else:
            self.date = now_date
            self.index = 0
            self.create_new_file()
        if text:
            level_text = MoVoidLog.log_level[level] if level in MoVoidLog.log_level else str(level)
            now_second = now_time.strftime("%H:%M:%S.%f")[:-3]
            real_text = '[{}][{}]{}'.format(now_second, level_text, text)
            print(real_text, file=self.file, flush=True)
            if print_out is True:
                print(real_text)
            elif print_out:
                if isinstance(print_out, list):
                    for print_file in print_out:
                        try:
                            print(real_text, file=print_file, flush=True)
                        except:
                            continue
                else:
                    try:
                        print(real_text, file=print_out, flush=True)
                    except:
                        print('we cannot print log text on <{}>:{}'.format(str(print_out), real_text), file=sys.stderr)
        self.check_file_delete()

    def create_new_file(self):
        if self.file is not None and not self.file.closed:
            self.file.close()
        if self.date is None:
            self.date = time.strftime("%Y-%m-%d", time.localtime(time.time()))
        if self.index < 0:
            self.index = 0
        new_path = os.path.join(self.dir, '{}-{}.{}.log'.format(self.name, self.date, self.index))
        if os.path.exists(new_path):
            self.find_file()
        else:
            self.file_path = new_path
            self.file = open(self.file_path, mode='w', encoding=self.encoding)


if __name__ == '__main__':
    a = MoVoidLog()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/27

"""
from setuptools import setup, find_packages

setup(
    name='movoid_tools',
    version='1.1.7',
    description='mo void create a lot of tools to use in many situations.',
    author='SunYiFan-MoVoid',
    author_email='bobrobotsun@163.com',
    license='MIT',
    packages=find_packages()
)

if __name__ == '__main__':
    pass
